/* *****************************************************************************
 * translations/mbtowc.c -- multi-byte -> fixed-width translating.
 * Copyright (C) 2017 Thomas "Cakeisalie5" Touhey <thomas@touhey.fr>
 *
 * This file is part of libfontcharacter.
 * libfontcharacter is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3.0 of the License,
 * or (at your option) any later version.
 *
 * libfontcharacter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libfontcharacter; if not, see <http://www.gnu.org/licenses/>.
 * ************************************************************************** */
#include <libfontcharacter/internals.h>
#include <string.h>

/**
 *	fc_mbtowc:
 *	Multi-byte to FONTCHARACTER character conversion.
 *
 *	Based on what is said in the fx-9860G SDK.
 *	Let's hope it stays true.
 *
 *	Based on how `mbtowc` works.
 *
 *	@arg	pfc		pointer to the FONTCHARACTER character.
 *	@arg	s		the multi-byte source string.
 *	@arg	n		the size of the destination buffer.
 *	@return			the number of bytes used (-1 if error).
 */

int fc_mbtowc(FONTCHARACTER *pfc, const char *s, size_t n)
{
	/* null string? */
	if (!s) return (-1);
	const unsigned char *us = (const unsigned char*)s;

	/* so that the 'no size = unlimited' (n == 0) case is easy to manage */
	n--;

	/* nul character? */
	if (!*us || *us == 0xFF)
		return (0);

	/* extended char? */
	if (*us && strchr("\x7F\xF7\xF9\xE5\xE6\xE7", *us)) {
		if (!n) return (-1);
		*pfc = *us;
		*pfc <<= 8;
		*pfc |= *(us + 1);
		return (2);
	}

	/* single-byte character! */
	*pfc = *us;
	return (1);
}

/**
 *	fc_mbstowcs:
 *	Multi-byte to FONTCHARACTER string conversion.
 *
 *	This function imitates `mbstowcs` (unicode), but for CASIO's encoding.
 *	You should be able to use it the same way.
 *
 *	@arg	dest	the destination.
 *	@arg	src		the source.
 *	@arg	n		the size of the source.
 *	@return			the size of the destination.
 */

size_t fc_mbstowcs(FONTCHARACTER *dest, const char *src, size_t n)
{
	size_t len = 0;
	n--; /* terminating character */

	while (n--) {
		/* read the FONTCHARACTER char */
		FONTCHARACTER wc;
		int count = fc_mbtowc(&wc, src, 0);
		if (count <= 0)
			break;

		/* write the entry and iterate */
		if (dest) *dest++ = wc;
		src += count;
		len++;
	}

	/* write the terminating entry and return length */
	if (dest) *dest = 0;
	return (len);
}
